@@ -1,6 +1,7 @@
-// Copyright (C) 2019-2021 The apca Developers
+// Copyright (C) 2019-2022 The apca Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
+use std::convert::TryFrom;
 use std::fmt::Display;
 use std::fmt::Formatter;
 use std::fmt::Result as FmtResult;
@@ -33,10 +34,20 @@ impl Deref for Id {
 
 /// An enumeration of the various asset classes available.
 #[derive(Clone, Copy, Debug, Deserialize, Serialize, PartialEq)]
+#[non_exhaustive]
 pub enum Class {
   /// US equities.
   #[serde(rename = "us_equity")]
   UsEquity,
+  /// Crypto currencies.
+  #[serde(rename = "crypto")]
+  Crypto,
+  /// Any other asset class that we have not accounted for.
+  ///
+  /// Note that having any such unknown asset class should be considered
+  /// a bug.
+  #[serde(other, rename(serialize = "unknown"))]
+  Unknown,
 }
 
 impl AsRef<str> for Class {
@@ -44,6 +55,8 @@ impl AsRef<str> for Class {
   fn as_ref(&self) -> &'static str {
     match *self {
       Class::UsEquity => "us_equity",
+      Class::Crypto => "crypto",
+      Class::Unknown => "unknown",
     }
   }
 }
@@ -58,10 +71,16 @@ impl Default for Class {
 impl FromStr for Class {
   type Err = ();
 
+  #[inline]
   fn from_str(s: &str) -> Result<Self, Self::Err> {
     if s == Class::UsEquity.as_ref() {
       Ok(Class::UsEquity)
+    } else if s == Class::Crypto.as_ref() {
+      Ok(Class::Crypto)
     } else {
+      // Note that we do not support creating the `Unknown` variant
+      // here. This variant is really only meant to cover
+      // deserialization.
       Err(())
     }
   }
@@ -125,8 +144,9 @@ impl Display for ParseSymbolError {
 }
 
 
-/// A symbol, and the various ways to represent it.
-#[derive(Clone, Debug, PartialEq)]
+/// A symbol and the various ways to represent it.
+#[derive(Clone, Debug, Deserialize, PartialEq)]
+#[serde(try_from = "&str")]
 pub enum Symbol {
   /// The symbol. Note that this is not a unique way to identify an
   /// asset (the same symbol may be used in different exchanges or asset
@@ -147,13 +167,11 @@ impl From<Id> for Symbol {
   }
 }
 
-impl<S> From<S> for Symbol
-where
-  S: Into<String>,
-{
-  #[inline]
-  fn from(symbol: S) -> Self {
-    Self::Sym(symbol.into())
+impl TryFrom<&str> for Symbol {
+  type Error = ParseSymbolError;
+
+  fn try_from(other: &str) -> Result<Self, Self::Error> {
+    Symbol::from_str(other)
   }
 }
 
@@ -221,7 +239,8 @@ impl Serialize for Symbol {
 
 
 /// An enumeration of the various supported exchanges.
-#[derive(Clone, Copy, Debug, Deserialize, PartialEq)]
+#[derive(Clone, Copy, Debug, Deserialize, PartialEq, Serialize)]
+#[non_exhaustive]
 pub enum Exchange {
   /// American Stock Exchange.
   #[serde(rename = "AMEX")]
@@ -242,9 +261,13 @@ pub enum Exchange {
   /// NYSE Arca.
   #[serde(rename = "NYSEARCA")]
   Nysearca,
+  /// An over-the-counter desk.
+  #[serde(rename = "OTC")]
+  Otc,
   /// Any other exchange that we have not accounted for.
   ///
-  /// Note that having any such status should be considered a bug.
+  /// Note that having any such unknown exchange should be considered a
+  /// bug.
   #[serde(other)]
   Unknown,
 }
@@ -258,6 +281,7 @@ impl AsRef<str> for Exchange {
       Exchange::Nasdaq => "NASDAQ",
       Exchange::Nyse => "NYSE",
       Exchange::Nysearca => "NYSEARCA",
+      Exchange::Otc => "OTC",
       Exchange::Unknown => "unknown",
     }
   }
@@ -362,6 +386,7 @@ mod tests {
   use crate::Client;
 
 
+  /// Verify that we can parse various symbols.
   #[test]
   fn parse_symbol() {
     let id = "b0b6dd9d-8b9b-48a9-ba46-b9d54906e415";
@@ -400,25 +425,32 @@ mod tests {
     );
   }
 
+  /// Make sure that we can serialize and deserialize a symbol.
   #[test]
-  fn serialize_symbol() {
+  fn serialize_deserialize_symbol() {
     let symbol = Symbol::Sym("AAPL".to_string());
-    assert_eq!(to_json(&symbol).unwrap(), r#""AAPL""#);
+    let json = to_json(&symbol).unwrap();
+    assert_eq!(json, r#""AAPL""#);
+    assert_eq!(from_json::<Symbol>(&json).unwrap(), symbol);
 
     let symbol = Symbol::SymExchg("AAPL".to_string(), Exchange::Nasdaq);
-    assert_eq!(to_json(&symbol).unwrap(), r#""AAPL:NASDAQ""#);
+    let json = to_json(&symbol).unwrap();
+    assert_eq!(json, r#""AAPL:NASDAQ""#);
+    assert_eq!(from_json::<Symbol>(&json).unwrap(), symbol);
 
     let symbol = Symbol::SymExchgCls("AAPL".to_string(), Exchange::Nasdaq, Class::UsEquity);
-    assert_eq!(to_json(&symbol).unwrap(), r#""AAPL:NASDAQ:us_equity""#);
+    let json = to_json(&symbol).unwrap();
+    assert_eq!(json, r#""AAPL:NASDAQ:us_equity""#);
+    assert_eq!(from_json::<Symbol>(&json).unwrap(), symbol);
 
     let id = Id(Uuid::parse_str("b0b6dd9d-8b9b-48a9-ba46-b9d54906e415").unwrap());
     let symbol = Symbol::Id(id);
-    assert_eq!(
-      to_json(&symbol).unwrap(),
-      r#""b0b6dd9d-8b9b-48a9-ba46-b9d54906e415""#
-    );
+    let json = to_json(&symbol).unwrap();
+    assert_eq!(json, r#""b0b6dd9d-8b9b-48a9-ba46-b9d54906e415""#);
+    assert_eq!(from_json::<Symbol>(&json).unwrap(), symbol);
   }
 
+  /// Check that we can parse a reference asset object.
   #[test]
   fn parse_reference_asset() {
     let response = r#"{
@@ -447,6 +479,7 @@ mod tests {
     assert!(asset.easy_to_borrow);
   }
 
+  /// Verify that we can parse an asset object with an unknown exchange.
   #[test]
   fn parse_with_unknown_exchange() {
     let response = r#"{
@@ -466,6 +499,7 @@ mod tests {
     assert_eq!(asset.exchange, Exchange::Unknown);
   }
 
+  /// Check that we can retrieve information about an asset.
   #[test(tokio::test)]
   async fn retrieve_asset() {
     async fn test(symbol: Symbol) {
