@@ -12,10 +12,8 @@ use http_endpoint::Bytes;
 
 use num_decimal::Num;
 
-use serde::ser::SerializeStruct;
 use serde::Deserialize;
 use serde::Serialize;
-use serde::Serializer;
 use serde_json::from_slice as from_json;
 use serde_json::to_vec as to_json;
 use serde_urlencoded::to_string as to_query;
@@ -28,7 +26,7 @@ use crate::Str;
 
 
 /// An ID uniquely identifying an order.
-#[derive(Clone, Copy, Debug, Deserialize, Eq, Hash, PartialEq)]
+#[derive(Clone, Copy, Debug, Deserialize, Eq, Hash, PartialEq, Serialize)]
 pub struct Id(pub Uuid);
 
 impl Deref for Id {
@@ -42,7 +40,7 @@ impl Deref for Id {
 
 
 /// The status an order can have.
-#[derive(Clone, Copy, Debug, Deserialize, PartialEq)]
+#[derive(Clone, Copy, Debug, Deserialize, PartialEq, Serialize)]
 pub enum Status {
   /// The order has been received by Alpaca, and routed to exchanges for
   /// execution. This is the usual initial state of an order.
@@ -123,6 +121,18 @@ pub enum Status {
   Unknown,
 }
 
+impl Status {
+  /// Check whether the status is terminal, i.e., no more changes will
+  /// occur to the associated order.
+  #[inline]
+  pub fn is_terminal(self) -> bool {
+    matches!(
+      self,
+      Self::Replaced | Self::Filled | Self::Canceled | Self::Expired | Self::Rejected
+    )
+  }
+}
+
 
 /// The side an order is on.
 #[derive(Clone, Copy, Debug, Deserialize, Serialize, PartialEq)]
@@ -238,33 +248,52 @@ impl Default for TimeInForce {
 }
 
 
+#[derive(Debug, Deserialize, Serialize)]
+#[serde(rename = "take_profit")]
+struct TakeProfitSerde {
+  #[serde(rename = "limit_price")]
+  limit_price: Num,
+}
+
+
 /// The take profit part of a bracket, one-cancels-other, or
 /// one-triggers-other order.
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
+#[serde(from = "TakeProfitSerde", into = "TakeProfitSerde")]
 pub enum TakeProfit {
   /// The limit price to use.
   Limit(Num),
 }
 
-impl Serialize for TakeProfit {
-  fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-  where
-    S: Serializer,
-  {
-    match self {
-      TakeProfit::Limit(limit) => {
-        let mut state = serializer.serialize_struct("take_profit", 1)?;
-        state.serialize_field("limit_price", &limit)?;
-        state.end()
-      },
+impl From<TakeProfitSerde> for TakeProfit {
+  fn from(other: TakeProfitSerde) -> Self {
+    Self::Limit(other.limit_price)
+  }
+}
+
+impl From<TakeProfit> for TakeProfitSerde {
+  fn from(other: TakeProfit) -> Self {
+    match other {
+      TakeProfit::Limit(limit_price) => Self { limit_price },
     }
   }
 }
 
 
+#[derive(Debug, Deserialize, Serialize)]
+#[serde(rename = "stop_loss")]
+struct StopLossSerde {
+  #[serde(rename = "stop_price")]
+  stop_price: Num,
+  #[serde(rename = "limit_price", skip_serializing_if = "Option::is_none")]
+  limit_price: Option<Num>,
+}
+
+
 /// The stop loss part of a bracket, one-cancels-other, or
 /// one-triggers-other order.
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
+#[serde(from = "StopLossSerde", into = "StopLossSerde")]
 pub enum StopLoss {
   /// The stop loss price to use.
   Stop(Num),
@@ -272,22 +301,26 @@ pub enum StopLoss {
   StopLimit(Num, Num),
 }
 
-impl Serialize for StopLoss {
-  fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-  where
-    S: Serializer,
-  {
-    match self {
-      StopLoss::Stop(stop) => {
-        let mut state = serializer.serialize_struct("stop_loss", 1)?;
-        state.serialize_field("stop_price", stop)?;
-        state.end()
+impl From<StopLossSerde> for StopLoss {
+  fn from(other: StopLossSerde) -> Self {
+    if let Some(limit_price) = other.limit_price {
+      Self::StopLimit(other.stop_price, limit_price)
+    } else {
+      Self::Stop(other.stop_price)
+    }
+  }
+}
+
+impl From<StopLoss> for StopLossSerde {
+  fn from(other: StopLoss) -> Self {
+    match other {
+      StopLoss::Stop(stop_price) => Self {
+        stop_price,
+        limit_price: None,
       },
-      StopLoss::StopLimit(stop, limit) => {
-        let mut state = serializer.serialize_struct("stop_loss", 2)?;
-        state.serialize_field("stop_price", stop)?;
-        state.serialize_field("limit_price", limit)?;
-        state.end()
+      StopLoss::StopLimit(stop_price, limit_price) => Self {
+        stop_price,
+        limit_price: Some(limit_price),
       },
     }
   }
@@ -364,12 +397,17 @@ pub struct OrderReqInit {
 
 impl OrderReqInit {
   /// Create an `OrderReq` from an `OrderReqInit`.
+  ///
+  /// The provided symbol is assumed to be a "simple" symbol and not any
+  /// of the composite forms of the [`Symbol`][asset::Symbol] enum. That
+  /// is, it is not being parsed but directly treated as the
+  /// [`Sym`][asset::Symbol::Sym] variant.
   pub fn init<S>(self, symbol: S, side: Side, amount: Amount) -> OrderReq
   where
-    S: Into<asset::Symbol>,
+    S: Into<String>,
   {
     OrderReq {
-      symbol: symbol.into(),
+      symbol: asset::Symbol::Sym(symbol.into()),
       amount,
       side,
       class: self.class,
@@ -389,7 +427,7 @@ impl OrderReqInit {
 
 
 /// A POST request to be made to the /v2/orders endpoint.
-#[derive(Clone, Debug, Serialize, PartialEq)]
+#[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
 pub struct OrderReq {
   /// Symbol or asset ID to identify the asset to trade.
   #[serde(rename = "symbol")]
@@ -499,8 +537,7 @@ pub struct ChangeReq {
 
 /// A single order as returned by the /v2/orders endpoint on a GET
 /// request.
-#[derive(Clone, Debug, Deserialize, PartialEq)]
-#[non_exhaustive]
+#[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
 pub struct Order {
   /// The order's ID.
   #[serde(rename = "id")]
@@ -760,7 +797,7 @@ mod tests {
 
   use futures::TryFutureExt;
 
-  use serde_json::from_str as from_json;
+  use serde_json::from_slice as from_json;
 
   use test_log::test;
 
@@ -775,18 +812,21 @@ mod tests {
   use crate::RequestError;
 
 
+  /// Check that we can serialize a [`Side`] object.
   #[test]
   fn emit_side() {
     assert_eq!(to_json(&Side::Buy).unwrap(), br#""buy""#);
     assert_eq!(to_json(&Side::Sell).unwrap(), br#""sell""#);
   }
 
+  /// Check that we can properly negate a [`Side`] object.
   #[test]
   fn negate_side() {
     assert_eq!(!Side::Buy, Side::Sell);
     assert_eq!(!Side::Sell, Side::Buy);
   }
 
+  /// Check that we can serialize a [`Type`] object.
   #[test]
   fn emit_type() {
     assert_eq!(to_json(&Type::Market).unwrap(), br#""market""#);
@@ -794,25 +834,30 @@ mod tests {
     assert_eq!(to_json(&Type::Stop).unwrap(), br#""stop""#);
   }
 
+  /// Make sure that we can serialize and deserialize order legs.
   #[test]
-  fn emit_legs() {
+  fn serialize_deserialize_legs() {
     let take_profit = TakeProfit::Limit(Num::new(3, 2));
-    let expected = br#"{"limit_price":"1.5"}"#;
-    assert_eq!(to_json(&take_profit).unwrap(), expected);
+    let json = to_json(&take_profit).unwrap();
+    assert_eq!(json, br#"{"limit_price":"1.5"}"#);
+    assert_eq!(from_json::<TakeProfit>(&json).unwrap(), take_profit);
 
     let stop_loss = StopLoss::Stop(Num::from(42));
-    let expected = br#"{"stop_price":"42"}"#;
-    assert_eq!(to_json(&stop_loss).unwrap(), expected);
+    let json = to_json(&stop_loss).unwrap();
+    assert_eq!(json, br#"{"stop_price":"42"}"#);
+    assert_eq!(from_json::<StopLoss>(&json).unwrap(), stop_loss);
 
     let stop_loss = StopLoss::StopLimit(Num::from(13), Num::from(96));
+    let json = to_json(&stop_loss).unwrap();
     let expected = br#"{"stop_price":"13","limit_price":"96"}"#;
-    assert_eq!(to_json(&stop_loss).unwrap(), &expected[..]);
+    assert_eq!(json, &expected[..]);
+    assert_eq!(from_json::<StopLoss>(&json).unwrap(), stop_loss);
   }
 
   /// Check that we can parse the `Amount::quantity` variant properly.
   #[test]
   fn parse_quantity_amount() {
-    let serialized = r#"{
+    let serialized = br#"{
     "qty": "15"
 }"#;
     let amount = from_json::<Amount>(serialized).unwrap();
@@ -822,16 +867,17 @@ mod tests {
   /// Check that we can parse the `Amount::notional` variant properly.
   #[test]
   fn parse_notional_amount() {
-    let serialized = r#"{
+    let serialized = br#"{
     "notional": "15.12"
 }"#;
     let amount = from_json::<Amount>(serialized).unwrap();
     assert_eq!(amount, Amount::notional(Num::from_str("15.12").unwrap()));
   }
 
+  /// Verify that we can deserialize and serialize a reference order.
   #[test]
-  fn parse_reference_order() {
-    let response = r#"{
+  fn deserialize_serialize_reference_order() {
+    let json = br#"{
     "id": "904837e3-3b76-47ec-b432-046db621571b",
     "client_order_id": "904837e3-3b76-47ec-b432-046db621571b",
     "created_at": "2018-10-05T05:48:59Z",
@@ -858,7 +904,7 @@ mod tests {
 }"#;
 
     let id = Id(Uuid::parse_str("904837e3-3b76-47ec-b432-046db621571b").unwrap());
-    let order = from_json::<Order>(response).unwrap();
+    let order = from_json::<Order>(&to_json(&from_json::<Order>(json).unwrap()).unwrap()).unwrap();
     assert_eq!(order.id, id);
     assert_eq!(
       order.created_at,
@@ -873,18 +919,41 @@ mod tests {
     assert_eq!(order.average_fill_price, Some(Num::new(10625, 100)));
   }
 
+  /// Check that we can serialize and deserialize an [`OrderReq`].
+  #[test]
+  fn serialize_deserialize_request() {
+    let request = OrderReqInit {
+      type_: Type::TrailingStop,
+      trail_price: Some(Num::from(50)),
+      ..Default::default()
+    }
+    .init("SPY", Side::Buy, Amount::quantity(1));
+
+    let json = to_json(&request).unwrap();
+    assert_eq!(from_json::<OrderReq>(&json).unwrap(), request);
+  }
+
   /// Verify that we can submit a limit order.
   #[test(tokio::test)]
   async fn submit_limit_order() {
     async fn test(extended_hours: bool) -> Result<(), RequestError<PostError>> {
       let symbol = Symbol::SymExchgCls("SPY".to_string(), Exchange::Arca, asset::Class::UsEquity);
-      let request = OrderReqInit {
+      let request = OrderReq {
+        symbol,
+        amount: Amount::quantity(1),
+        side: Side::Buy,
+        class: Class::default(),
         type_: Type::Limit,
+        time_in_force: TimeInForce::default(),
         limit_price: Some(Num::from(1)),
+        stop_price: None,
+        trail_price: None,
+        trail_percent: None,
+        take_profit: None,
+        stop_loss: None,
         extended_hours,
-        ..Default::default()
-      }
-      .init(symbol, Side::Buy, Amount::quantity(1));
+        client_order_id: None,
+      };
 
       let api_info = ApiInfo::from_env().unwrap();
       let client = Client::new(api_info);
@@ -921,13 +990,12 @@ mod tests {
   /// Check that we can properly submit a trailing stop price order.
   #[test(tokio::test)]
   async fn submit_trailing_stop_price_order() {
-    let symbol = Symbol::Sym("SPY".to_string());
     let request = OrderReqInit {
       type_: Type::TrailingStop,
       trail_price: Some(Num::from(50)),
       ..Default::default()
     }
-    .init(symbol, Side::Buy, Amount::quantity(1));
+    .init("SPY", Side::Buy, Amount::quantity(1));
 
     let api_info = ApiInfo::from_env().unwrap();
     let client = Client::new(api_info);
@@ -950,13 +1018,12 @@ mod tests {
   /// Check that we can properly submit a trailing stop percent order.
   #[test(tokio::test)]
   async fn submit_trailing_stop_percent_order() {
-    let symbol = Symbol::Sym("SPY".to_string());
     let request = OrderReqInit {
       type_: Type::TrailingStop,
       trail_percent: Some(Num::from(10)),
       ..Default::default()
     }
-    .init(symbol, Side::Buy, Amount::quantity(1));
+    .init("SPY", Side::Buy, Amount::quantity(1));
 
     let api_info = ApiInfo::from_env().unwrap();
     let client = Client::new(api_info);
