@@ -39,9 +39,9 @@ use crate::websocket::MessageResult;
 use crate::Error;
 
 
-/// The status of a trade, as reported as part of a `TradeUpdate`.
-#[derive(Clone, Copy, Debug, Deserialize, PartialEq)]
-pub enum TradeStatus {
+/// The status of an order, as reported as part of a `OrderUpdate`.
+#[derive(Clone, Copy, Debug, Deserialize, PartialEq, Serialize)]
+pub enum OrderStatus {
   /// The order has been received by Alpaca, and routed to exchanges for
   /// execution.
   #[serde(rename = "new")]
@@ -110,10 +110,11 @@ pub enum TradeStatus {
 
 /// An enumeration of the different event streams.
 #[derive(Copy, Clone, Debug, Deserialize, PartialEq, Serialize)]
-enum StreamType {
-  /// A stream for trade updates.
+#[doc(hidden)]
+pub enum StreamType {
+  /// A stream for order updates.
   #[serde(rename = "trade_updates")]
-  TradeUpdates,
+  OrderUpdates,
 }
 
 
@@ -122,7 +123,7 @@ enum StreamType {
 #[doc(hidden)]
 pub struct Streams<'d> {
   /// A list of stream types.
-  streams: Cow<'d, [StreamType]>,
+  pub streams: Cow<'d, [StreamType]>,
 }
 
 impl<'d> From<&'d [StreamType]> for Streams<'d> {
@@ -136,8 +137,10 @@ impl<'d> From<&'d [StreamType]> for Streams<'d> {
 
 
 /// The status reported in authentication control messages.
-#[derive(Debug, Deserialize, PartialEq)]
-enum AuthenticationStatus {
+#[derive(Debug, Deserialize, PartialEq, Serialize)]
+#[doc(hidden)]
+#[allow(missing_copy_implementations)]
+pub enum AuthenticationStatus {
   /// The client has been authorized.
   #[serde(rename = "authorized")]
   Authorized,
@@ -149,12 +152,12 @@ enum AuthenticationStatus {
 
 /// The authentication related data provided in authentication control
 /// messages.
-#[derive(Debug, Deserialize)]
+#[derive(Debug, Deserialize, Serialize)]
 #[doc(hidden)]
 pub struct Authentication {
   /// The status of an operation.
   #[serde(rename = "status")]
-  status: AuthenticationStatus,
+  pub status: AuthenticationStatus,
   /*
    * TODO: Right now we just ignore the `action` field, as we would
    *       not react on it anyway.
@@ -163,19 +166,20 @@ pub struct Authentication {
 
 
 /// A control message "request" sent over a websocket channel.
-#[derive(Debug, Serialize)]
+#[derive(Debug, Deserialize, Serialize)]
+#[doc(hidden)]
 #[serde(tag = "action", content = "data")]
-enum Request<'d> {
-  /// A control message indicating whether or not we were authenticated
-  /// successfully.
+pub enum Request<'d> {
+  /// A request to authenticate with the server after a websocket
+  /// connection was established.
   #[serde(rename = "authenticate")]
   Authenticate {
     #[serde(rename = "key_id")]
-    key_id: &'d str,
+    key_id: Cow<'d, str>,
     #[serde(rename = "secret_key")]
-    secret: &'d str,
+    secret: Cow<'d, str>,
   },
-  /// A control message detailing the streams we are subscribed to.
+  /// A request to subscribe to a particular stream.
   #[serde(rename = "listen")]
   Listen(Streams<'d>),
 }
@@ -195,14 +199,14 @@ pub enum ControlMessage {
 
 /// An enum representing the different messages we may receive over our
 /// websocket channel.
-#[derive(Debug, Deserialize)]
+#[derive(Debug, Deserialize, Serialize)]
 #[doc(hidden)]
 #[serde(tag = "stream", content = "data")]
 #[allow(clippy::large_enum_variant)]
-pub enum TradeMessage {
-  /// A trade update.
+pub enum OrderMessage {
+  /// An order update.
   #[serde(rename = "trade_updates")]
-  TradeUpdate(TradeUpdate),
+  OrderUpdate(OrderUpdate),
   /// A control message indicating whether or not we were authenticated
   /// successfully.
   #[serde(rename = "authorization")]
@@ -213,37 +217,36 @@ pub enum TradeMessage {
 }
 
 
-/// A representation of a trade update that we receive through the
+/// A representation of an order update that we receive through the
 /// "trade_updates" stream.
-#[derive(Clone, Debug, Deserialize, PartialEq)]
-#[non_exhaustive]
-pub struct TradeUpdate {
+#[derive(Clone, Debug, Deserialize, PartialEq, Serialize)]
+pub struct OrderUpdate {
   /// The event that occurred.
   #[serde(rename = "event")]
-  pub event: TradeStatus,
-  /// The order associated with the trade.
+  pub event: OrderStatus,
+  /// The order that received an update.
   #[serde(rename = "order")]
   pub order: order::Order,
 }
 
 
 /// A websocket message that we tried to parse.
-type ParsedMessage = MessageResult<Result<TradeMessage, JsonError>, WebSocketError>;
+type ParsedMessage = MessageResult<Result<OrderMessage, JsonError>, WebSocketError>;
 
 impl subscribe::Message for ParsedMessage {
-  type UserMessage = Result<Result<TradeUpdate, JsonError>, WebSocketError>;
+  type UserMessage = Result<Result<OrderUpdate, JsonError>, WebSocketError>;
   type ControlMessage = ControlMessage;
 
   fn classify(self) -> subscribe::Classification<Self::UserMessage, Self::ControlMessage> {
     match self {
       MessageResult::Ok(Ok(message)) => match message {
-        TradeMessage::TradeUpdate(update) => subscribe::Classification::UserMessage(Ok(Ok(update))),
-        TradeMessage::AuthenticationMessage(authentication) => {
+        OrderMessage::OrderUpdate(update) => subscribe::Classification::UserMessage(Ok(Ok(update))),
+        OrderMessage::AuthenticationMessage(authentication) => {
           subscribe::Classification::ControlMessage(ControlMessage::AuthenticationMessage(
             authentication,
           ))
         },
-        TradeMessage::ListeningMessage(streams) => {
+        OrderMessage::ListeningMessage(streams) => {
           subscribe::Classification::ControlMessage(ControlMessage::ListeningMessage(streams))
         },
       },
@@ -266,7 +269,7 @@ impl subscribe::Message for ParsedMessage {
 }
 
 
-/// A subscription allowing certain control operations pertaining trade
+/// A subscription allowing certain control operations pertaining order
 /// update retrieval.
 #[derive(Debug)]
 pub struct Subscription<S>(subscribe::Subscription<S, ParsedMessage, wrap::Message>);
@@ -281,7 +284,10 @@ where
     key_id: &str,
     secret: &str,
   ) -> Result<Result<(), Error>, S::Error> {
-    let request = Request::Authenticate { key_id, secret };
+    let request = Request::Authenticate {
+      key_id: key_id.into(),
+      secret: secret.into(),
+    };
     let json = match to_json(&request) {
       Ok(json) => json,
       Err(err) => return Ok(Err(Error::Json(err))),
@@ -308,9 +314,9 @@ where
     }
   }
 
-  /// Subscribe and listen to trade updates.
+  /// Subscribe and listen to order updates.
   async fn listen(&mut self) -> Result<Result<(), Error>, S::Error> {
-    let streams = Streams::from([StreamType::TradeUpdates].as_ref());
+    let streams = Streams::from([StreamType::OrderUpdates].as_ref());
     let request = Request::Listen(streams);
     let json = match to_json(&request) {
       Ok(json) => json,
@@ -322,9 +328,9 @@ where
     match response {
       Some(response) => match response {
         Ok(ControlMessage::ListeningMessage(streams)) => {
-          if !streams.streams.contains(&StreamType::TradeUpdates) {
+          if !streams.streams.contains(&StreamType::OrderUpdates) {
             return Ok(Err(Error::Str(
-              "server did not subscribe us to trade update stream".into(),
+              "server did not subscribe us to order update stream".into(),
             )))
           }
           Ok(Ok(()))
@@ -333,7 +339,7 @@ where
           "server responded with an unexpected message".into(),
         ))),
         Err(()) => Ok(Err(Error::Str(
-          "failed to listen to trade update stream".into(),
+          "failed to listen to order update stream".into(),
         ))),
       },
       None => Ok(Err(Error::Str(
@@ -351,10 +357,10 @@ type MapFn = fn(Result<wrap::Message, WebSocketError>) -> ParsedMessage;
 /// A type used for requesting a subscription to the "trade_updates"
 /// event stream.
 #[derive(Clone, Copy, Debug, PartialEq)]
-pub enum TradeUpdates {}
+pub enum OrderUpdates {}
 
 #[async_trait]
-impl Subscribable for TradeUpdates {
+impl Subscribable for OrderUpdates {
   type Input = ApiInfo;
   type Subscription = Subscription<SplitSink<Stream, wrap::Message>>;
   type Stream = Fuse<MessageStream<SplitStream<Stream>, ParsedMessage>>;
@@ -362,31 +368,19 @@ impl Subscribable for TradeUpdates {
   async fn connect(api_info: &Self::Input) -> Result<(Self::Stream, Self::Subscription), Error> {
     fn map(result: Result<wrap::Message, WebSocketError>) -> ParsedMessage {
       MessageResult::from(result.map(|message| match message {
-        wrap::Message::Text(string) => json_from_str::<TradeMessage>(&string),
-        wrap::Message::Binary(data) => json_from_slice::<TradeMessage>(&data),
+        wrap::Message::Text(string) => json_from_str::<OrderMessage>(&string),
+        wrap::Message::Binary(data) => json_from_slice::<OrderMessage>(&data),
       }))
     }
 
     let ApiInfo {
-      base_url: url,
+      api_stream_url: url,
       key_id,
       secret,
+      ..
     } = api_info;
 
-    let mut url = url.clone();
-    // TODO: We really shouldn't need this conditional logic. Find a
-    //       better way.
-    match url.scheme() {
-      "ws" | "wss" => (),
-      _ => {
-        url.set_scheme("wss").map_err(|()| {
-          Error::Str(format!("unable to change URL scheme for {}: invalid URL?", url).into())
-        })?;
-        url.set_path("stream");
-      },
-    }
-
-    let stream = connect(&url).await?.map(map as MapFn);
+    let stream = connect(url).await?.map(map as MapFn);
     let (send, recv) = stream.split();
     let (stream, subscription) = subscribe::subscribe(recv, send);
     let mut stream = stream.fuse();
@@ -431,8 +425,6 @@ mod tests {
 
   use test_log::test;
 
-  use url::Url;
-
   use websocket_util::test::WebSocketStream;
   use websocket_util::tungstenite::error::ProtocolError;
   use websocket_util::tungstenite::Message;
@@ -460,8 +452,8 @@ mod tests {
   /// Check that we can encode an authentication request correctly.
   #[test]
   fn encode_authentication_request() {
-    let key_id = "some-key";
-    let secret = "super-secret-secret";
+    let key_id = "some-key".into();
+    let secret = "super-secret-secret".into();
     let expected = {
       r#"{"action":"authenticate","data":{"key_id":"some-key","secret_key":"super-secret-secret"}}"#
     };
@@ -475,15 +467,15 @@ mod tests {
   #[test]
   fn encode_listen_request() {
     let expected = r#"{"action":"listen","data":{"streams":["trade_updates"]}}"#;
-    let streams = Streams::from([StreamType::TradeUpdates].as_ref());
+    let streams = Streams::from([StreamType::OrderUpdates].as_ref());
     let request = Request::Listen(streams);
     let json = to_json(&request).unwrap();
     assert_eq!(json, expected)
   }
 
-  /// Verify that we can decode a trade update.
+  /// Verify that we can decode an order update.
   #[test]
-  fn decode_trade_update() {
+  fn decode_order_update() {
     let json = r#"{
   "stream":"trade_updates","data":{
     "event":"new","execution_id":"11111111-2222-3333-4444-555555555555","order":{
@@ -501,10 +493,10 @@ mod tests {
     },"timestamp":"2021-12-09T19:48:46.182987144Z"
   }
 }"#;
-    let message = json_from_str::<TradeMessage>(json).unwrap();
+    let message = json_from_str::<OrderMessage>(json).unwrap();
     match message {
-      TradeMessage::TradeUpdate(update) => {
-        assert_eq!(update.event, TradeStatus::New);
+      OrderMessage::OrderUpdate(update) => {
+        assert_eq!(update.event, OrderStatus::New);
         assert_eq!(update.order.side, order::Side::Buy);
       },
       _ => panic!("Decoded unexpected message variant: {:?}", message),
@@ -516,9 +508,9 @@ mod tests {
   fn decode_authentication() {
     let json =
       { r#"{"stream":"authorization","data":{"status":"authorized","action":"authenticate"}}"# };
-    let message = json_from_str::<TradeMessage>(json).unwrap();
+    let message = json_from_str::<OrderMessage>(json).unwrap();
     match message {
-      TradeMessage::AuthenticationMessage(authentication) => {
+      OrderMessage::AuthenticationMessage(authentication) => {
         assert_eq!(authentication.status, AuthenticationStatus::Authorized);
       },
       _ => panic!("Decoded unexpected message variant: {:?}", message),
@@ -531,9 +523,9 @@ mod tests {
   fn decode_unauthorized_authentication() {
     let json =
       { r#"{"stream":"authorization","data":{"status":"unauthorized","action":"listen"}}"# };
-    let message = json_from_str::<TradeMessage>(json).unwrap();
+    let message = json_from_str::<OrderMessage>(json).unwrap();
     match message {
-      TradeMessage::AuthenticationMessage(authentication) => {
+      OrderMessage::AuthenticationMessage(authentication) => {
         assert_eq!(authentication.status, AuthenticationStatus::Unauthorized);
       },
       _ => panic!("Decoded unexpected message variant: {:?}", message),
@@ -545,10 +537,10 @@ mod tests {
   fn decode_listening() {
     let json = r#"{"stream":"listening","data":{"streams":["trade_updates"]}}"#;
 
-    let message = json_from_str::<TradeMessage>(json).unwrap();
+    let message = json_from_str::<OrderMessage>(json).unwrap();
     match message {
-      TradeMessage::ListeningMessage(streams) => {
-        assert_eq!(streams.streams, vec![StreamType::TradeUpdates]);
+      OrderMessage::ListeningMessage(streams) => {
+        assert_eq!(streams.streams, vec![StreamType::OrderUpdates]);
       },
       _ => panic!("Decoded unexpected message variant: {:?}", message),
     }
@@ -565,7 +557,7 @@ mod tests {
       Ok(())
     }
 
-    let result = mock_stream::<TradeUpdates, _, _>(test).await;
+    let result = mock_stream::<OrderUpdates, _, _>(test).await;
     match result {
       Ok(..) => panic!("authentication succeeded unexpectedly"),
       Err(Error::WebSocket(WebSocketError::Protocol(e)))
@@ -596,7 +588,7 @@ mod tests {
       Ok(())
     }
 
-    let result = mock_stream::<TradeUpdates, _, _>(test).await;
+    let result = mock_stream::<OrderUpdates, _, _>(test).await;
     match result {
       Ok(..) => panic!("operation succeeded unexpectedly"),
       Err(Error::Str(ref e)) if e.starts_with("stream was closed before listen") => (),
@@ -605,7 +597,7 @@ mod tests {
   }
 
   /// Check that we can correctly handle a successful subscription
-  /// without trade update messages.
+  /// without order update messages.
   #[test(tokio::test)]
   async fn no_messages() {
     async fn test(mut stream: WebSocketStream) -> Result<(), WebSocketError> {
@@ -625,7 +617,7 @@ mod tests {
       Ok(())
     }
 
-    let err = mock_stream::<TradeUpdates, _, _>(test).await.unwrap_err();
+    let err = mock_stream::<OrderUpdates, _, _>(test).await.unwrap_err();
     match err {
       Error::WebSocket(WebSocketError::Protocol(e))
         if e == ProtocolError::ResetWithoutClosingHandshake => {},
@@ -651,7 +643,7 @@ mod tests {
       Ok(())
     }
 
-    let result = mock_stream::<TradeUpdates, _, _>(test).await.unwrap_err();
+    let result = mock_stream::<OrderUpdates, _, _>(test).await.unwrap_err();
     match result {
       Error::Json(_) => (),
       e => panic!("received unexpected error: {}", e),
@@ -691,7 +683,7 @@ mod tests {
       }
     };
 
-    let (stream, _subscription) = mock_stream::<TradeUpdates, _, _>(test).await.unwrap();
+    let (stream, _subscription) = mock_stream::<OrderUpdates, _, _>(test).await.unwrap();
     let () = sender.send(()).unwrap();
 
     stream
@@ -728,7 +720,7 @@ mod tests {
       Ok(())
     }
 
-    let (stream, _subscription) = mock_stream::<TradeUpdates, _, _>(test).await.unwrap();
+    let (stream, _subscription) = mock_stream::<OrderUpdates, _, _>(test).await.unwrap();
     stream
       .map_err(Error::from)
       .try_for_each(|_| ready(Ok(())))
@@ -736,57 +728,52 @@ mod tests {
       .unwrap();
   }
 
-  /// Test the end-to-end workflow of streaming a trade update for a
+  /// Test the end-to-end workflow of streaming an order update for a
   /// newly created order.
   #[test(tokio::test)]
-  async fn stream_trade_events() {
+  async fn stream_order_events() {
     // TODO: There may be something amiss here. If we don't cancel the
-    //       order we never get an event about a new trade. That does
+    //       order we never get an event about a new order. That does
     //       not seem to be in our code, though, as the behavior is the
     //       same when streaming events using Alpaca's Python client.
     let api_info = ApiInfo::from_env().unwrap();
     let client = Client::new(api_info);
-    let (stream, _subscription) = client.subscribe::<TradeUpdates>().await.unwrap();
+    let (stream, _subscription) = client.subscribe::<OrderUpdates>().await.unwrap();
 
     let order = order_aapl(&client).await.unwrap();
     client.issue::<order::Delete>(&order.id).await.unwrap();
 
-    let trade = stream
-      .try_filter_map(|res| {
-        let trade = res.unwrap();
-        ok(Some(trade))
+    let update = stream
+      .try_filter_map(|result| {
+        let update = result.unwrap();
+        ok(Some(update))
       })
-      // There could be other trades happening concurrently but we
+      // There could be other orders happening concurrently but we
       // are only interested in ones belonging to the order we
       // submitted as part of this test.
-      .try_skip_while(|trade| ok(trade.order.id != order.id))
+      .try_skip_while(|update| ok(update.order.id != order.id))
       .next()
       .await
       .unwrap()
       .unwrap();
 
-    assert_eq!(order.id, trade.order.id);
-    assert_eq!(order.asset_id, trade.order.asset_id);
-    assert_eq!(order.symbol, trade.order.symbol);
-    assert_eq!(order.asset_class, trade.order.asset_class);
-    assert_eq!(order.type_, trade.order.type_);
-    assert_eq!(order.side, trade.order.side);
-    assert_eq!(order.time_in_force, trade.order.time_in_force);
+    assert_eq!(order.id, update.order.id);
+    assert_eq!(order.asset_id, update.order.asset_id);
+    assert_eq!(order.symbol, update.order.symbol);
+    assert_eq!(order.asset_class, update.order.asset_class);
+    assert_eq!(order.type_, update.order.type_);
+    assert_eq!(order.side, update.order.side);
+    assert_eq!(order.time_in_force, update.order.time_in_force);
   }
 
   /// Test that we fail as expected when attempting to authenticate for
-  /// trade updates using invalid credentials.
+  /// order updates using invalid credentials.
   #[test(tokio::test)]
   async fn stream_with_invalid_credentials() {
-    let api_base = Url::parse(API_BASE_URL).unwrap();
-    let api_info = ApiInfo {
-      base_url: api_base,
-      key_id: "invalid".to_string(),
-      secret: "invalid-too".to_string(),
-    };
+    let api_info = ApiInfo::from_parts(API_BASE_URL, "invalid", "invalid-too").unwrap();
 
     let client = Client::new(api_info);
-    let err = client.subscribe::<TradeUpdates>().await.unwrap_err();
+    let err = client.subscribe::<OrderUpdates>().await.unwrap_err();
 
     match err {
       Error::Str(ref e) if e == "authentication not successful" => (),
