@@ -8,6 +8,7 @@ use serde::Deserialize;
 use serde::Serialize;
 use serde_urlencoded::to_string as to_query;
 
+use crate::data::v2::Feed;
 use crate::data::DATA_BASE_URL;
 use crate::util::vec_from_str;
 use crate::Str;
@@ -36,7 +37,9 @@ pub struct Quotes {
 #[derive(Clone, Debug, Default, PartialEq)]
 pub struct QuotesReqInit {
   /// See `QuotesReq::limit`.
-  pub limit: Option<u64>,
+  pub limit: Option<usize>,
+  /// See `QuotesReq::feed`.
+  pub feed: Option<Feed>,
   /// See `QuotesReq::page_token`.
   pub page_token: Option<String>,
   #[doc(hidden)]
@@ -55,6 +58,7 @@ impl QuotesReqInit {
       start,
       end,
       limit: self.limit,
+      feed: self.feed,
       page_token: self.page_token,
     }
   }
@@ -79,7 +83,10 @@ pub struct QuotesReq {
   /// Number of quotes to return. Must be in range 1-10000, defaults to
   /// 1000.
   #[serde(rename = "limit")]
-  pub limit: Option<u64>,
+  pub limit: Option<usize>,
+  /// The data feed to use.
+  #[serde(rename = "feed")]
+  pub feed: Option<Feed>,
   /// Pagination token to continue from.
   #[serde(rename = "page_token")]
   pub page_token: Option<String>,
@@ -96,7 +103,7 @@ Endpoint! {
   ],
   Err => GetError, [
     /// Some of the provided data was invalid or not found.
-    /* 422 */ UNPROCESSABLE_ENTITY => InvalidValues,
+    /* 422 */ UNPROCESSABLE_ENTITY => InvalidInput,
   ]
 
   fn base_url() -> Option<Str> {
@@ -164,7 +171,7 @@ mod tests {
     let request = QuotesReqInit::default().init("ABC123", start, end);
     let err = client.issue::<Get>(&request).await.unwrap_err();
     match err {
-      RequestError::Endpoint(GetError::InvalidValues(_)) => (),
+      RequestError::Endpoint(GetError::InvalidInput(_)) => (),
       _ => panic!("Received unexpected error: {:?}", err),
     };
   }
@@ -186,7 +193,7 @@ mod tests {
 
     let err = client.issue::<Get>(&request).await.unwrap_err();
     match err {
-      RequestError::Endpoint(GetError::InvalidValues(_)) => (),
+      RequestError::Endpoint(GetError::InvalidInput(_)) => (),
       _ => panic!("Received unexpected error: {:?}", err),
     };
   }
