@@ -1,4 +1,4 @@
-// Copyright (C) 2021 The apca Developers
+// Copyright (C) 2021-2022 The apca Developers
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 use std::borrow::Borrow as _;
@@ -32,13 +32,13 @@ use serde_json::from_str as json_from_str;
 use serde_json::to_string as to_json;
 use serde_json::Error as JsonError;
 
+use thiserror::Error as ThisError;
+
 use tokio::net::TcpStream;
 
 use tungstenite::MaybeTlsStream;
 use tungstenite::WebSocketStream;
 
-use url::Url;
-
 use websocket_util::subscribe;
 use websocket_util::subscribe::MessageStream;
 use websocket_util::tungstenite::Error as WebSocketError;
@@ -47,7 +47,6 @@ use websocket_util::wrap::Wrapper;
 
 use super::unfold::Unfold;
 
-use crate::data::DATA_WEBSOCKET_BASE_URL;
 use crate::subscribable::Subscribable;
 use crate::websocket::connect;
 use crate::websocket::MessageResult;
@@ -78,6 +77,9 @@ mod private {
 
 
 /// A trait representing the source from which to stream real time data.
+// TODO: Once we can use enumerations as const generic parameters we
+//       should probably switch over to repurposing `data::v2::Feed`
+//       here instead.
 pub trait Source: private::Sealed {
   /// Return a textual representation of the source.
   #[doc(hidden)]
@@ -93,6 +95,7 @@ pub trait Source: private::Sealed {
 pub enum IEX {}
 
 impl Source for IEX {
+  #[inline]
   fn as_str() -> &'static str {
     "iex"
   }
@@ -109,6 +112,7 @@ impl private::Sealed for IEX {}
 pub enum SIP {}
 
 impl Source for SIP {
+  #[inline]
   fn as_str() -> &'static str {
     "sip"
   }
@@ -148,6 +152,7 @@ pub enum Symbol {
 }
 
 impl From<&'static str> for Symbol {
+  #[inline]
   fn from(symbol: &'static str) -> Self {
     if symbol == "*" {
       Symbol::All
@@ -158,6 +163,7 @@ impl From<&'static str> for Symbol {
 }
 
 impl From<String> for Symbol {
+  #[inline]
   fn from(symbol: String) -> Self {
     if symbol == "*" {
       Symbol::All
@@ -261,9 +267,34 @@ pub struct Bar {
 }
 
 
-/// An error as reported by the Alpaca Data API.
+/// A quote for an equity.
 #[derive(Clone, Debug, Deserialize, PartialEq)]
-pub struct ApiError {
+pub struct Quote {
+  /// The quote's symbol.
+  #[serde(rename = "S")]
+  pub symbol: String,
+  /// The bid's price.
+  #[serde(rename = "bp")]
+  pub bid_price: Num,
+  /// The bid's size.
+  #[serde(rename = "bs")]
+  pub bid_size: u64,
+  /// The ask's price.
+  #[serde(rename = "ap")]
+  pub ask_price: Num,
+  /// The ask's size.
+  #[serde(rename = "as")]
+  pub ask_size: u64,
+  /// The quote's time stamp.
+  #[serde(rename = "t")]
+  pub timestamp: DateTime<Utc>,
+}
+
+
+/// An error as reported by the Alpaca Stream API.
+#[derive(Clone, Debug, Deserialize, PartialEq, ThisError)]
+#[error("{message} ({code})")]
+pub struct StreamApiError {
   /// The error code being reported.
   #[serde(rename = "code")]
   pub code: u64,
@@ -283,6 +314,9 @@ pub enum DataMessage {
   /// A variant representing aggregate data for a given symbol.
   #[serde(rename = "b")]
   Bar(Bar),
+  /// A variant representing a quote for a given symbol.
+  #[serde(rename = "q")]
+  Quote(Quote),
   /// A control message describing the current list of subscriptions.
   #[serde(rename = "subscription")]
   Subscription(MarketData),
@@ -290,17 +324,34 @@ pub enum DataMessage {
   /// successful.
   #[serde(rename = "success")]
   Success,
-  /// An error reported by the Alpaca Data API.
+  /// An error reported by the Alpaca Stream API.
   #[serde(rename = "error")]
-  Error(ApiError),
+  Error(StreamApiError),
 }
 
 
 /// A data item as received over the our websocket channel.
 #[derive(Debug)]
+#[non_exhaustive]
 pub enum Data {
   /// A variant representing aggregate data for a given symbol.
   Bar(Bar),
+  /// A variant representing quote data for a given symbol.
+  Quote(Quote),
+}
+
+impl Data {
+  /// Check whether this object is of the `Bar` variant.
+  #[inline]
+  pub fn is_bar(&self) -> bool {
+    matches!(self, Self::Bar(..))
+  }
+
+  /// Check whether this object is of the `Quote` variant.
+  #[inline]
+  pub fn is_quote(&self) -> bool {
+    matches!(self, Self::Quote(..))
+  }
 }
 
 
@@ -313,8 +364,8 @@ pub enum ControlMessage {
   /// A control message indicating that the last operation was
   /// successful.
   Success,
-  /// An error reported by the Alpaca Data API.
-  Error(ApiError),
+  /// An error reported by the Alpaca Stream API.
+  Error(StreamApiError),
 }
 
 
@@ -329,6 +380,9 @@ impl subscribe::Message for ParsedMessage {
     match self {
       MessageResult::Ok(Ok(message)) => match message {
         DataMessage::Bar(bar) => subscribe::Classification::UserMessage(Ok(Ok(Data::Bar(bar)))),
+        DataMessage::Quote(quote) => {
+          subscribe::Classification::UserMessage(Ok(Ok(Data::Quote(quote))))
+        },
         DataMessage::Subscription(data) => {
           subscribe::Classification::ControlMessage(ControlMessage::Subscription(data))
         },
@@ -347,7 +401,7 @@ impl subscribe::Message for ParsedMessage {
   #[inline]
   fn is_error(user_message: &Self::UserMessage) -> bool {
     // Both outer `WebSocketError` and inner `JsonError` errors
-    // constitute errors in our sense. Note however than an API error
+    // constitute errors in our sense. Note, however, that an API error
     // does not. It's just a regular control message from our
     // perspective.
     user_message
@@ -359,6 +413,7 @@ impl subscribe::Message for ParsedMessage {
 
 
 /// Deserialize a normalized [`Symbols`] object from a string.
+#[inline]
 fn normalized_from_str<'de, D>(deserializer: D) -> Result<Symbols, D::Error>
 where
   D: Deserializer<'de>,
@@ -373,12 +428,14 @@ where
 pub struct Normalized(#[serde(deserialize_with = "normalized_from_str")] Symbols);
 
 impl From<Symbols> for Normalized {
+  #[inline]
   fn from(symbols: Symbols) -> Self {
     Self(normalize(symbols))
   }
 }
 
 impl From<Vec<String>> for Normalized {
+  #[inline]
   fn from(symbols: Vec<String>) -> Self {
     Self(normalize(Cow::from(
       IntoIterator::into_iter(symbols)
@@ -389,6 +446,7 @@ impl From<Vec<String>> for Normalized {
 }
 
 impl<const N: usize> From<[&'static str; N]> for Normalized {
+  #[inline]
   fn from(symbols: [&'static str; N]) -> Self {
     Self(normalize(Cow::from(
       IntoIterator::into_iter(symbols)
@@ -403,18 +461,33 @@ impl<const N: usize> From<[&'static str; N]> for Normalized {
 #[derive(Clone, Debug, Default, Deserialize, PartialEq, Serialize)]
 pub struct MarketData {
   /// The aggregate bars to subscribe to.
+  #[serde(default)]
   pub bars: Normalized,
+  /// The quotes to subscribe to.
+  #[serde(default)]
+  pub quotes: Normalized,
 }
 
 impl MarketData {
   /// A convenience function for setting the [`bars`][MarketData::bars]
   /// member.
+  #[inline]
   pub fn set_bars<N>(&mut self, symbols: N)
   where
     N: Into<Normalized>,
   {
     self.bars = symbols.into();
   }
+
+  /// A convenience function for setting the [`quotes`][MarketData::quotes]
+  /// member.
+  #[inline]
+  pub fn set_quotes<N>(&mut self, symbols: N)
+  where
+    N: Into<Normalized>,
+  {
+    self.quotes = symbols.into();
+  }
 }
 
 
@@ -461,6 +534,7 @@ pub struct Subscription<S> {
 
 impl<S> Subscription<S> {
   /// Create a `Subscription` object wrapping the `websocket_util` based one.
+  #[inline]
   fn new(subscription: subscribe::Subscription<S, ParsedMessage, wrap::Message>) -> Self {
     Self {
       subscription,
@@ -526,6 +600,9 @@ where
           self.subscriptions = data;
           Ok(Ok(()))
         },
+        Ok(ControlMessage::Error(error)) => Ok(Err(Error::Str(
+          format!("failed to subscribe: {}", error).into(),
+        ))),
         Ok(_) => Ok(Err(Error::Str(
           "server responded with unexpected message".into(),
         ))),
@@ -542,6 +619,7 @@ where
   /// Contained in `subscribe` are the *additional* symbols to subscribe
   /// to. Use the [`unsubscribe`][Self::unsubscribe] method to
   /// unsubscribe from receiving data for certain symbols.
+  #[inline]
   pub async fn subscribe(&mut self, subscribe: &MarketData) -> Result<Result<(), Error>, S::Error> {
     let request = Request::Subscribe(subscribe);
     self.subscribe_unsubscribe(&request).await
@@ -551,6 +629,7 @@ where
   ///
   /// Subscriptions of market data for symbols other than the ones
   /// provide to this function are left untouched.
+  #[inline]
   pub async fn unsubscribe(
     &mut self,
     unsubscribe: &MarketData,
@@ -560,6 +639,7 @@ where
   }
 
   /// Inquire the currently active individual market data subscriptions.
+  #[inline]
   pub fn subscriptions(&self) -> &MarketData {
     &self.subscriptions
   }
@@ -604,27 +684,14 @@ where
     }
 
     let ApiInfo {
-      base_url: url,
+      data_stream_base_url: url,
       key_id,
       secret,
+      ..
     } = api_info;
 
     let mut url = url.clone();
-    // For (some) testing purposes we may want to connect to a local URL
-    // (from a mock server) and we identify these cases as those that
-    // already have a proper websocket scheme present. For all others,
-    // we just use the "live" data URL we have hard coded.
-    // TODO: We really shouldn't need this conditional logic. Find a
-    //       better way.
-    match url.scheme() {
-      "ws" | "wss" => (),
-      _ => {
-        // We basically only work statically defined URL parts here which we
-        // know can be parsed successfully, so unwrapping is fine.
-        url = Url::parse(DATA_WEBSOCKET_BASE_URL).unwrap();
-        url.set_path(&format!("v2/{}", S::as_str()));
-      },
-    }
+    url.set_path(&format!("v2/{}", S::as_str()));
 
     let stream =
       Unfold::new(connect(&url).await?.map(parse as ParseFn)).map(MessageResult::from as MapFn);
@@ -703,8 +770,10 @@ mod tests {
   //       `crate::websocket::test::SECRET` here.
   const AUTH_REQ: &str = r#"{"action":"auth","key":"USER12345678","secret":"justletmein"}"#;
   const AUTH_RESP: &str = r#"[{"T":"success","msg":"authenticated"}]"#;
-  const SUB_REQ: &str = r#"{"action":"subscribe","bars":["AAPL","VOO"]}"#;
+  const SUB_REQ: &str = r#"{"action":"subscribe","bars":["AAPL","VOO"],"quotes":[]}"#;
   const SUB_RESP: &str = r#"[{"T":"subscription","bars":["AAPL","VOO"]}]"#;
+  const SUB_ERR_REQ: &str = r#"{"action":"subscribe","bars":[],"quotes":[]}"#;
+  const SUB_ERR_RESP: &str = r#"[{"T":"error","code":400,"msg":"invalid syntax"}]"#;
 
 
   /// Check that we can deserialize the [`DataMessage::Bar`] variant.
@@ -738,6 +807,42 @@ mod tests {
     );
   }
 
+  /// Check that we can deserialize the [`DataMessage::Quote`] variant.
+  #[test]
+  fn parse_quote() {
+    let json: &str = r#"{
+  "T": "q",
+  "S": "NVDA",
+  "bx": "P",
+  "bp": 258.8,
+  "bs": 2,
+  "ax": "A",
+  "ap": 259.99,
+  "as": 5,
+  "c": [
+      "R"
+  ],
+  "z": "C",
+  "t": "2022-01-18T23:09:42.151875584Z"
+}"#;
+
+    let message = json_from_str::<DataMessage>(json).unwrap();
+    let quote = match message {
+      DataMessage::Quote(qoute) => qoute,
+      _ => panic!("Decoded unexpected message variant: {:?}", message),
+    };
+    assert_eq!(quote.symbol, "NVDA");
+    assert_eq!(quote.bid_price, Num::new(2588, 10));
+    assert_eq!(quote.bid_size, 2);
+    assert_eq!(quote.ask_price, Num::new(25999, 100));
+    assert_eq!(quote.ask_size, 5);
+
+    assert_eq!(
+      quote.timestamp,
+      Utc.ymd(2022, 1, 18).and_hms_nano(23, 9, 42, 151875584)
+    );
+  }
+
   /// Check that we can deserialize the [`DataMessage::Success`] variant.
   #[test]
   fn parse_success() {
@@ -796,7 +901,7 @@ mod tests {
     let request = Request::Subscribe(&data);
 
     let json = to_json(&request).unwrap();
-    let expected = r#"{"action":"subscribe","bars":["AAPL","VOO"]}"#;
+    let expected = r#"{"action":"subscribe","bars":["AAPL","VOO"],"quotes":[]}"#;
     assert_eq!(json, expected);
   }
 
@@ -809,7 +914,7 @@ mod tests {
     let request = Request::Unsubscribe(&data);
 
     let json = to_json(&request).unwrap();
-    let expected = r#"{"action":"unsubscribe","bars":["VOO"]}"#;
+    let expected = r#"{"action":"unsubscribe","bars":["VOO"],"quotes":[]}"#;
     assert_eq!(json, expected);
   }
 
@@ -856,7 +961,7 @@ mod tests {
   }
 
   /// Check that we can correctly handle a successful subscription
-  /// without trade update messages.
+  /// without pushing actual data.
   #[test(tokio::test)]
   async fn authenticate_and_subscribe() {
     async fn test(mut stream: WebSocketStream) -> Result<(), WebSocketError> {
@@ -898,6 +1003,47 @@ mod tests {
       .unwrap();
   }
 
+  /// Check that we correctly handle errors reported as part of
+  /// subscription.
+  #[test(tokio::test)]
+  async fn subscribe_error() {
+    async fn test(mut stream: WebSocketStream) -> Result<(), WebSocketError> {
+      stream.send(Message::Text(CONN_RESP.to_string())).await?;
+      // Authentication.
+      assert_eq!(
+        stream.next().await.unwrap()?,
+        Message::Text(AUTH_REQ.to_string()),
+      );
+      stream.send(Message::Text(AUTH_RESP.to_string())).await?;
+
+      // Subscription.
+      assert_eq!(
+        stream.next().await.unwrap()?,
+        Message::Text(SUB_ERR_REQ.to_string()),
+      );
+      stream.send(Message::Text(SUB_ERR_RESP.to_string())).await?;
+      stream.send(Message::Close(None)).await?;
+      Ok(())
+    }
+
+    let (mut stream, mut subscription) =
+      mock_stream::<RealtimeData<IEX>, _, _>(test).await.unwrap();
+
+    let data = MarketData::default();
+
+    let subscribe = subscription.subscribe(&data).boxed_local().fuse();
+    let error = drive(subscribe, &mut stream)
+      .await
+      .unwrap()
+      .unwrap()
+      .unwrap_err();
+
+    match error {
+      Error::Str(ref e) if e == "failed to subscribe: invalid syntax (400)" => {},
+      e => panic!("received unexpected error: {}", e),
+    }
+  }
+
   /// Check that we can adjust the current market data subscription on
   /// the fly.
   #[test(tokio::test)]
@@ -957,7 +1103,51 @@ mod tests {
 
     let read = stream
       .map_err(Error::WebSocket)
-      .try_for_each(|result| async { result.map(|_data| ()).map_err(Error::Json) });
+      .try_for_each(|result| async {
+        result
+          .map(|data| {
+            assert!(data.is_bar());
+          })
+          .map_err(Error::Json)
+      });
+
+    if timeout(Duration::from_millis(100), read).await.is_ok() {
+      panic!("realtime data stream got exhausted unexpectedly")
+    }
+  }
+
+  /// Check that we can stream realtime stock quotes.
+  ///
+  /// Note that we do not have any control over whether the market is
+  /// open or not and as such we can only try on a best-effort basis to
+  /// receive and decode updates.
+  #[test(tokio::test)]
+  #[serial(realtime_data)]
+  async fn stream_quotes() {
+    let api_info = ApiInfo::from_env().unwrap();
+    let client = Client::new(api_info);
+    let (mut stream, mut subscription) = client.subscribe::<RealtimeData<IEX>>().await.unwrap();
+
+    let mut data = MarketData::default();
+    data.set_quotes(["SPY"]);
+
+    let subscribe = subscription.subscribe(&data).boxed_local().fuse();
+    let () = drive(subscribe, &mut stream)
+      .await
+      .unwrap()
+      .unwrap()
+      .unwrap();
+
+    let read = stream
+      .map_err(Error::WebSocket)
+      .try_for_each(|result| async {
+        result
+          .map(|data| {
+            assert!(data.is_quote());
+          })
+          .map_err(Error::Json)
+      });
+
     if timeout(Duration::from_millis(100), read).await.is_ok() {
       panic!("realtime data stream got exhausted unexpectedly")
     }
@@ -992,13 +1182,7 @@ mod tests {
   #[test(tokio::test)]
   #[serial(realtime_data)]
   async fn stream_with_invalid_credentials() {
-    let api_base = Url::parse(API_BASE_URL).unwrap();
-    let api_info = ApiInfo {
-      base_url: api_base,
-      key_id: "invalid".to_string(),
-      secret: "invalid-too".to_string(),
-    };
-
+    let api_info = ApiInfo::from_parts(API_BASE_URL, "invalid", "invalid-too").unwrap();
     let client = Client::new(api_info);
     let err = client.subscribe::<RealtimeData<IEX>>().await.unwrap_err();
 
