@@ -47,7 +47,7 @@
   while_true
 )]
 
-//! A create for interacting with the Alpaca API.
+//! A crate for interacting with the Alpaca API.
 
 #[macro_use]
 extern crate http_endpoint;
@@ -73,6 +73,7 @@ use std::borrow::Cow;
 
 pub use crate::api_info::ApiInfo;
 pub use crate::client::Client;
+pub use crate::endpoint::ApiError;
 pub use crate::error::Error;
 pub use crate::error::RequestError;
 pub use crate::subscribable::Subscribable;
